/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "plasma.h"

#ifdef TUNED_ASSUMPTION
// Annahme: nebeneinanderliegende Pixel sind im Speicher nacheinander!

void virt2gfx(int phase, int size, int * virtual, \
 EA_PIXEL * ziel, EA_PIXEL * rainbow )
// Virtuellen 8bpp Schirm anzeigen - ZEITKRITISCH
{
if ( (sizeof(EA_PIXEL)==2) && (sizeof(int)==4) )
{ register int y;
  for (y=0; y<size; y++)
  { __asm__ __volatile__
    ("cld
      pushl %%eax
      pushl %%edi
      .align 16
      1:
      lodsl                     # read *virtual, virtual++
      addl %%edx,%%eax          # add phase
      movzbl %%al,%%eax         # mask to 8 bit - type A
      # andl $255,%%eax         # mask to 8 bit - type B
      # movzbl ist besser, da der Loop dann in 16 Byte passt.
      # Zumindest auf Cx6x68... andere CPUs haben evtl. AL/EAX-Clash-Penalties
      movw (%%ebx,%%eax,2),%%ax # read rainbow[...]
      stosw                     # write *gfxp, gfxp++
      cmpl %%ecx,%%edi          # gfxp high enough?
      jb 1b
      popl %%edi
      popl %%eax
     ":"=S"(virtual):"0"(virtual),"b"(rainbow),"d"(phase),
        "D"(&GFX(0,y,ziel)),"c"(&GFX(size,y,ziel))
    );
  }; // Das war der entscheidende Schritt zur reellen Geschwindigkeit!
}
else
{ register int x;
  register int y;
  register int y2;
  err_exit("Plasma - Tried to be fast, but data size was wrong\n");
  for (y=0, y2=0; y<size; y++)
  { for (x=0; x<size; x++, y2++)
    { GFX( x , y , ziel ) =
      rainbow[ 255 & ( phase + virtual[y2] ) ];
     };
   }; // Das war der entscheidende Schritt zur virtuellen Palettenanimation!
};

}; // NICHT inline, da sonst nicht genug reg-vars verfuegbar sind!

#else

void virt2gfx(int phase, int size, int * virtual, \
 EA_PIXEL * ziel, EA_PIXEL * rainbow )
// Virtuellen 8bpp Schirm anzeigen - ZEITKRITISCH
{ register int x;
  register int y;
  register int y2;
  for (y=0, y2=0; y<size; y++)
  { for (x=0; x<size; x++, y2++)
    { GFX( x , y , ziel ) =
      rainbow[ 255 & ( phase + virtual[y2] ) ];
     };
   }; // Das war der entscheidende Schritt zur virtuellen Palettenanimation!
}; // NICHT inline, da sonst nicht genug reg-vars verfuegbar sind!

#endif
